import requests
import ollama


class OllamaClient:
    def __init__(self,model_name):
        self.client = ollama.Client()
        self.model = model_name

    def query_xpath(self, html, query):
        prompt = f"""
        Given the following HTML and a natural language query, return the XPath for the described element.

        HTML:
        {html[:10000]}

        Query: {query}

        XPath:
        """
        response = self.client.generate(model=self.model,prompt=prompt)
        
        if "response" in response:
            output = response["response"]
            xpath = output.split("XPath:")[-1].strip()
            return xpath
        else:
            raise ValueError("Failed to get response from Ollama model.")

        
        
class OllamaServerClient:
    def __init__(self, model_name="llama3", base_url="http://localhost:11434/api/generate"):
        self.model = model_name
        self.base_url = base_url

    def query_xpath(self, html, query):
        prompt = f"""
        Given the following HTML and a natural language query, return the XPath for the described element.

        HTML:
        {html[:10000]}

        Query: {query}

        XPath:
        """

        response = requests.post(
            self.base_url,
            json={
                "model": self.model,
                "prompt": prompt,
                "stream": False
            }
        )

        if response.status_code == 200:
            data = response.json()
            output = data.get("response", "")
            xpath = output.split("XPath:")[-1].strip()
            return xpath
        else:
            raise Exception(f"Ollama error ({response.status_code}): {response.text}")



from setuptools import setup, find_packages

with open('README.md','r') as f:
    description = f.read()

setup(
    name="selenia",
    version="0.2.0",
    description="Selenium wrapper to find XPath from natural language using LLMs",
    long_description=description,
    long_description_content_type='text/markdown',
    author="Penielny",
    author_email="penielnyinaku@gmail.com",
    packages=find_packages(),
    install_requires=[
        "selenium",
        "requests",
        "ollama",
    ],
    python_requires=">=3.7",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)



from selenium.webdriver.common.by import By

class Selenia:
    def __init__(self, driver, model_client):
        self.driver = driver
        self.model_client = model_client

    def find(self, description: str):
        """Find an element using a natural language description."""
        html = self.driver.page_source
        xpath = self.model_client.query_xpath(html, description)
        return self.driver.find_element(By.XPATH, xpath)

    def find_all(self, description: str):
        """Find all matching elements using a natural language description."""
        html = self.driver.page_source
        xpath = self.model_client.query_xpath(html, description)
        return self.driver.find_elements(By.XPATH, xpath)
